@@ -6,5 +6,6 @@ func ServiceManagerStartNotify() error {
 	return nil
 }
 
-func ServiceManagerReadyNotify() {
+func ServiceManagerReadyNotify() error {
+	return nil
 }
